local RelicLogic = require("relic.RelicLogic")

function CG_RELIC_QUERY(human,msg)
    RelicLogic.relicQuery(human,msg.camp)
end

function CG_RELIC_UNLOCK_UNLOCK(human,msg)
    RelicLogic.unlockDo(human,msg.camp,msg.inputIDList,msg.inputIndexList)
end

function CG_RELIC_LVUP_DO(human,msg)
    RelicLogic.lvUpDo(human,msg.camp)
end

function CG_RELIC_JINJIE_QUERY(human,msg)   
    RelicLogic.jinjieQuery(human,msg.camp)
end

function CG_RELIC_JINJIE_YULAN_QUERY(human,msg)
    RelicLogic.yulanQuery(human,msg.camp)
end

function CG_RELIC_TUPO_DO(human,msg)
    RelicLogic.jinjieDo(human,msg.camp)
end 